@@ -1774,7 +1774,7 @@ static qsizetype toLatin1Len(qsizetype l) { return l + 1; }
     operation, encoding UTF-16 encoded data (usually in the form of a QString) to
     the requested encoding.
 
-    The supported encodings are:
+    The following encodings are always supported:
 
     \list
     \li UTF-8
@@ -1788,6 +1788,10 @@ static qsizetype toLatin1Len(qsizetype l) { return l + 1; }
     \li The system encoding
     \endlist
 
+    QStringConverter may support more encodings depending on how Qt was
+    compiled. If more codecs are supported, they can be listed using
+    availableCodecs().
+
     \l {QStringConverter}s can be used as follows to convert some encoded
     string to and from UTF-16.
 
@@ -1954,7 +1958,7 @@ struct QStringConverterICU : QStringConverter
         const void *context;
         ucnv_getToUCallBack(icu_conv, &action, &context);
         if (context != state)
-             ucnv_setToUCallBack(icu_conv, action, &state, nullptr, nullptr, &err);
+             ucnv_setToUCallBack(icu_conv, action, state, nullptr, nullptr, &err);
 
         ucnv_toUnicode(icu_conv, &target, targetLimit, &source, sourceLimit, nullptr, flush, &err);
         // We did reserve enough space:
@@ -1987,7 +1991,7 @@ struct QStringConverterICU : QStringConverter
         const void *context;
         ucnv_getFromUCallBack(icu_conv, &action, &context);
         if (context != state)
-             ucnv_setFromUCallBack(icu_conv, action, &state, nullptr, nullptr, &err);
+             ucnv_setFromUCallBack(icu_conv, action, state, nullptr, nullptr, &err);
 
         ucnv_fromUnicode(icu_conv, &target, targetLimit, &source, sourceLimit, nullptr, flush, &err);
         // We did reserve enough space:
@@ -2356,6 +2360,10 @@ static qsizetype availableCodecCount()
     QStringDecoder's constructor to create a en- or decoder for
     the given codec.
 
+    This function may be used to obtain a listing of additional codecs beyond
+    the standard ones. Support for additional codecs requires Qt be compiled
+    with support for the ICU library.
+
     \note The order of codecs is an internal implementation detail
     and not guaranteed to be stable.
  */
