@@ -16,7 +16,6 @@
 #include <qpa/qplatformservices.h>
 
 #include <QtGui/private/qguiapplication_p.h>
-#include <QtGui/private/qhighdpiscaling_p.h>
 
 #include <QtGui/qguiapplication.h>
 
@@ -547,6 +546,11 @@ static void populateFromPattern(FcPattern *pattern,
 
 }
 
+static bool isDprScaling()
+{
+    return !qFuzzyCompare(qApp->devicePixelRatio(), 1.0);
+}
+
 QFontconfigDatabase::~QFontconfigDatabase()
 {
     FcConfigDestroy(FcConfigGetCurrent());
@@ -651,7 +655,7 @@ QFontEngine::HintStyle defaultHintStyleFromMatch(QFont::HintingPreference hintin
         break;
     }
 
-    if (QHighDpiScaling::isActive())
+    if (isDprScaling())
         return QFontEngine::HintNone;
 
     int hint_style = 0;
@@ -958,7 +962,7 @@ QFont QFontconfigDatabase::defaultFont() const
 void QFontconfigDatabase::setupFontEngine(QFontEngineFT *engine, const QFontDef &fontDef) const
 {
     bool antialias = !(fontDef.styleStrategy & QFont::NoAntialias);
-    bool forcedAntialiasSetting = !antialias || QHighDpiScaling::isActive();
+    bool forcedAntialiasSetting = !antialias || isDprScaling();
 
     const QPlatformServices *services = QGuiApplicationPrivate::platformIntegration()->services();
     bool useXftConf = false;
